@@ -1,22 +1,22 @@
-""" 
+"""
     Module to handle an interface to modeller, used to rebuild main and side chains and to optimize side chain orientation
 """
 
 import sys
 import os
+from os.path import join as opj
 import uuid
 import shutil
-from os.path import join as opj
 
 from Bio import SeqIO, pairwise2
-from Bio.SeqRecord import SeqRecord
 from Bio.Seq import Seq
+from Bio.SeqRecord import SeqRecord
 
 try:
-    from modeller import *
-    from modeller.automodel import *
-except:
-    sys.exit("Error importing modeller")
+    from modeller import Environ, log
+    from modeller.automodel import automodel, assess
+except ImportError:
+    sys.exit("Error importing Modeller package")
 
 # Check for back-compatiblity with biopython < 1.77
 try:
@@ -29,11 +29,11 @@ TMP_BASE_DIR = '/tmp'
 DEBUG = False
 
 class ModellerManager():
-    """ 
+    """
     | modeller_manager ModellerManager
     | Class to handle Modeller calculations """
     def __init__(self):
-        self.tmpdir = TMP_BASE_DIR +  "/mod" + str(uuid.uuid4())
+        self.tmpdir = opj(TMP_BASE_DIR, "mod" + str(uuid.uuid4()))
         #self.tmpdir = "/tmp/modtest"
         #print("Using temporary working dir " + self.tmpdir)
         self.ch_id = ''
@@ -55,7 +55,7 @@ class ModellerManager():
     def build(self, target_model, target_chain, extra_NTerm_res):
         """ ModellerManager.build
         Prepare Modeller input and builds the model
-        
+
         Args:
             target_model (int) : Model to repair
             target_chain (str) : Chain to repair
@@ -77,7 +77,6 @@ class ModellerManager():
 
         templs = []
         knowns = []
-
         for ch_id in self.sequences.data[target_chain]['chains']:
             frgs = self.sequences.data[ch_id]['pdb'][target_model]['frgs']
             pdb_seq = frgs[0].seq
@@ -158,7 +157,7 @@ def _write_alin(tgt_seq, templs, alin_file):
     )
 
 class NoCanSeqError(Exception):
-    """ 
+    """
     | modeller_manager NoCanSeqError
     | Error raised when no canonical sequence exists
     """
